#include <string>
#include <map>
#include "../../../../../include/Order2Go2.h"
#include "sessionstatus.h"
#include "responselistener.h"

int main(int argc, char *argv[])
{
    const char *instrument = 0;
    const char *tf = 0;
    const char *user = 0;
    const char *password = 0;
    const char *url = 0;
    const char *connection = 0;
    const char *subsession = 0;
    const char *pin = 0;
    if (argc < 4)
    {
        printf("usage: login.exe instrument timeframe user password url connection subsession? pin?");
        printf("       timeframe: t1, m1, m5, m15, m30, H1, H2, H4, H6, H8, H12, D1, W1, M1");
        return -1;
    }

    instrument = argv[1];
    tf = argv[2];
    user = argv[3];
    password = argv[4];
    if (argc >= 6)
        url = argv[5];
    else
        url = "http://www.fxcorporate.com/Hosts.jsp";
    if (argc >= 7)
        connection = argv[6];
    else
        connection = "Demo";

    IO2GSession *session = CO2GTransport::createSession();
    SessionStatusListener *status_listener = new SessionStatusListener(session, true, subsession, pin);
    ResponseListener *response_listener = new ResponseListener(session);
    session->subscribeSessionStatus(status_listener);
    session->subscribeResponse(response_listener);
    status_listener->resetError();
    printf("connecting...\n");
    session->login(user, password, url, connection);

    bool connected = false;

    while (true)
    {
        uni::WaitForSingleObject(status_listener->getEvent(), 250);
        if (status_listener->getStatusCode() == IO2GSessionStatus::Connected)
        {
            connected = true;
            break;
        }
        if (status_listener->wasError())
        {
            printf("Connection failed: %s", status_listener->getError());
            break;
        }
    }

    if (connected)
    {
        printf("connected\n");

        //find time frame object


        //send market data snapshot request
        IO2GRequestFactory *factory = session->getRequestFactory();

        //find time frame by identifier
        IO2GTimeframeCollection *timeframe_collection = factory->getTimeFrameCollection();
        IO2GTimeframe *timeframe = 0;
        bool found = false;
        int size;
        size = timeframe_collection->size();
        for (int i = 0; i < size; i++)
        {
            IO2GTimeframe *timeframe1 = timeframe_collection->get(i);
            if (strcmp(timeframe1->getID(), tf) == 0)
            {
                timeframe = timeframe1;
                found = true;
                break;
            }
            else
                timeframe1->release();
        }

        if (found)
        {
            //get 300 last candles of the chosen instrument in chosen time frame
            //note: to choose data range use factory->fillMarketDataSnapshotRequestTime(...)  method
            IO2GRequest *request = factory->createMarketDataSnapshotRequestInstrument(instrument, timeframe, 300);
            response_listener->setRequest(request->getRequestID());
            session->sendRequest(request);
            timeframe->release();
            request->release();
            factory->release();

            while (!response_listener->isFinished())
                Sleep(50);      //for faster detection of request completion use events as shown in the
                                //SessionStatus listener example
        }
        else
            printf("timeframe %s if not found\n", tf);

        session->logout();
        printf("disconnecting...\n");
        while (true)
        {
            uni::WaitForSingleObject(status_listener->getEvent(), 250);
            if (status_listener->getStatusCode() == IO2GSessionStatus::Disconnected)
                break;
        }
        printf("disconnected\n");
    }

    session->unsubscribeResponse(response_listener);
    response_listener->release();
    session->unsubscribeSessionStatus(status_listener);
    status_listener->release();
    session->release();
}

