#include <math.h>
#include <time.h>
#include <string>
#include <vector>
#include <map>
#include "../../../../../include/Order2Go2.h"
#include "responselistener.h"
#include "sessionstatus.h"

ResponseListener::ResponseListener(IO2GSession *session)
{
    mSession = session;
    mSession->addRef();
    mRefCount = 1;
}

ResponseListener::~ResponseListener()
{
    mSession->release();
}

/** Increase reference counter. */
long ResponseListener::addRef()
{
    return InterlockedIncrement(&mRefCount);
}

/** Decrease reference counter. */
long ResponseListener::release()
{
    long rc = InterlockedDecrement(&mRefCount);
    if (rc == 0)
        delete this;
    return rc;
}

/** Set request. */
void ResponseListener::setRequest(const char *requestId)
{
    mRequestID = requestId;
    mFinished = false;
}

bool ResponseListener::isFinished() const
{
    return mFinished;
}


/** Request execution completed data handler. */
void ResponseListener::onRequestCompleted(const char *requestId, IO2GResponse  *response)
{
    if (response != 0)
    {
        if (response->getType() == MarketDataSnapshot)
        {
            //process updates
            IO2GResponseReaderFactory *factory = mSession->getResponseReaderFactory();
            if (factory == 0)
                return;

            IO2GTimeConverter *timeConverter = mSession->getTimeConverter();
            IO2GMarketDataSnapshotResponseReader *reader = factory->createMarketDataSnapshotReader(response);
            if (reader != 0)
            {
                int size = reader->size();
                bool isBar = reader->isBar();
                for (int i = 0; i < size; i++)
                {
                    char stime[64];

                    DATE dt = timeConverter->convert(reader->getDate(i), IO2GTimeConverter::UTC, IO2GTimeConverter::Local);
                    formatDate(dt, stime);

                    if (isBar)
                    {
                        printf("%s %f %f %f %f/%f %f %f %f/%i\n", stime,
                               reader->getBidOpen(i), reader->getBidHigh(i), reader->getBidLow(i), reader->getBidClose(i),
                               reader->getAskOpen(i), reader->getAskHigh(i), reader->getAskLow(i), reader->getAskClose(i),
                               reader->getVolume(i));
                    }
                    else
                    {
                        printf("%s %f/%f\n", stime, reader->getBid(i), reader->getAsk(i));
                    }

                }
                reader->release();
            }
            factory->release();
        }
    }

    if (requestId != 0 && mRequestID.length() > 0 && strcmp(requestId, mRequestID.c_str()) == 0)
        mFinished = true;
}

/** Request execution failed data handler. */
void ResponseListener::onRequestFailed(const char *requestId , const char *error)
{
    printf("Request %s failed:\n%s\n", requestId, error);
    if (requestId != 0 && mRequestID.length() > 0 && strcmp(requestId, mRequestID.c_str()) == 0)
        mFinished = true;
}

/** Request update data received data handler. */
void ResponseListener::onTablesUpdates(IO2GResponse *data)
{
}

/** Format date to print */
void ResponseListener::formatDate(DATE d, char *buf)
{
    double d_int, d_frac;
    d_frac = modf(d, &d_int);
    time_t t = time_t(d_int - 25569.0) * 86400 + time_t(floor(d_frac * 86400));
    struct tm *t1 = gmtime(&t);
    sprintf(buf, "%04i/%02i/%02i %02i:%02i:%02i", t1->tm_year + 1900, t1->tm_mon + 1, t1->tm_mday, t1->tm_hour, t1->tm_min, t1->tm_sec);
}
