/*
 * Header: AThread.h
 *
 * Copyright (c) 2002-2003, FXCM, LLC.
 * Financial Square 32 Old Slip, 10th Floor, New York, NY 10005 USA
 * 
 * THIS SOFTWARE IS THE CONFIDENTIAL AND PROPRIETARY INFORMATION OF 
 * FXCM, LLC. ("CONFIDENTIAL INFORMATION"). YOU SHALL NOT DISCLOSE 
 * SUCH CONFIDENTIAL INFORMATION AND SHALL USE IT ONLY IN ACCORDANCE
 * WITH THE TERMS OF THE LICENSE AGREEMENT YOU ENTERED INTO WITH 
 * FXCM.
 *    
 * File description: Incapsulates unified OS thread support
 *
 * $History: $
 *  
 * Jan 12, 2011     nmyltsev   Initial created.
 */

#pragma once

#if !defined(WIN32) && !defined(PTHREADS)
#define PTHREADS
#endif

#ifdef PTHREADS
#include "pthread.h"
#endif

#include "Mutex.h"

#ifndef INFINITE
#define INFINITE        0xFFFFFFFF    // Infinite timeout
#endif
#define THREAD_OBJECT_HAS_DELETED 0xFFFFFFFE
namespace threading
{

class AThread
{
 public:

    /** Class constructor

    Construct class, not start an associated new thread.
    New thread starts from run() method.

    */
    AThread();
    virtual ~AThread();

    virtual bool start();

    virtual bool join(unsigned long dwWaitMilliseconds = INFINITE);

    void requestStop()
    { 
        m_isStopRequested = true; 
    }

    bool isStopRequested() const
    { 
        return m_isStopRequested; 
    }

    bool isRunning() const
    {
        Mutex::Lock lock(m_hLock);
#ifdef PTHREADS
        return mRunning;
#else
        return mThreadID != 0; 
#endif
    }

protected:

    void resetRunning()
    {
#ifdef PTHREADS
    mRunning = false;
#else
    mThreadID = 0;
#endif
    }

    virtual int run(void *) = 0;
#ifdef PTHREADS
    static void *threadRunner(void *);
#else
    static unsigned int WINAPI threadRunner(void *);
#endif

#ifdef PTHREADS
    pthread_t m_oThread;
    bool mRunning;
#else
    unsigned int mThreadID;
    HANDLE m_hThread;
#endif  
    
    mutable Mutex m_hLock;
    volatile bool m_isStopRequested;
};

} //namespace
