//******************************************************************************
// @file          CWinEventHandle.h
// @copyright     GNU Library General Public License
// @version       $Header$
// @author        Richard Dengler
// @language      C++ ANSI V3
// @description   A Wrapper around pthread_cond_t. Used to emulate win32-events.
//******************************************************************************
#ifndef CWINEVENTHANDLE_H
#define CWINEVENTHANDLE_H
#include <pthread.h>
#include <string>
#include <set>

/*******************************************************************************
Opaque, polymorphic HANDLE class. Base class. Used internally.
*******************************************************************************/
class CBaseHandle
{
public:
  CBaseHandle() {}
  virtual ~CBaseHandle() {}
  virtual bool wait(unsigned numMs) { return false; }
  virtual bool close() { return false; }
};

/*******************************************************************************
Event HANDLE class. Used internally.
*******************************************************************************/
class CWinEventHandle : public CBaseHandle
{
public:
  CWinEventHandle(bool manualReset = false, bool signaled = false, const wchar_t* name = NULL);
  virtual ~CWinEventHandle();
  bool close();
  inline void incRefCount() { m_RefCount++; }
  inline int decRefCount() { m_RefCount--; return m_RefCount; }
  bool signaled() const { return m_Signaled; }
  std::wstring name() const { return m_Name; }
  void reset();
  void signal();
  bool pulse();
  bool wait(unsigned numMs);
  bool wait();
  void subscribe(CWinEventHandle* subscriber);
  void unSubscribe(CWinEventHandle* subscriber);
  bool isManualReset() const { return m_ManualReset; }
  void resetIfAuto();
private:
  pthread_mutex_t m_Mutex;
  pthread_mutex_t m_SubscrMutex;
  pthread_cond_t m_Cond;
  bool m_ManualReset;
  bool m_Signaled;
  int m_Count;
  int m_RefCount;
  std::wstring m_Name;
  std::set<CWinEventHandle*> m_Subscriber; // Used in WaitForMultipleObjects()
};

#endif

