package createoto;

import java.util.concurrent.ConcurrentLinkedQueue;
import com.fxcore2.*;

public class EventListener {
    private SimpleLog mSimpleLog;
    private ConcurrentLinkedQueue<O2GResponse> mResponseQueue;
    private O2GSession mSession;
    private EventPipe mEventPipe;

    class ParamsFromLoginRules {
        String sAccountID;
        String sOfferID;
        int iBaseAmount;
        double dPointSize;
        double dAskRate;
        double dBidRate;
    }
    public EventListener(SimpleLog simpleLog, ConcurrentLinkedQueue<O2GResponse> responseQueue, O2GSession session, EventPipe eventPipe) {
        mSimpleLog = simpleLog;
        mResponseQueue = responseQueue;
        mSession = session;
        mEventPipe = eventPipe;
    }

    public void waitNextEvent() {
        boolean bContinue = true;
        while (bContinue) {
            EventType eEventType = mEventPipe.waitEvent();
            if (eEventType == EventType.STATUS) {
                this.onStatusChanged();
            } else if (eEventType == EventType.CONNECTED) {
                this.onConnected();
            } else if (eEventType == EventType.RESPONSE) {
                this.onResponse();
            } else {
                // "How to login" : Logout when you are done.
                mSession.logout();
                bContinue = false;
            }
        }
    }

    private void onResponse() {
        while (true) {
            if (mResponseQueue.size() == 0)
                break;

            O2GResponse response = mResponseQueue.poll();

        }
    }

    private void onConnected() {
        prepareAndCreateOTO(); 
    }

    private void onStatusChanged() {
    }

    private void prepareAndCreateOTO()
    {
        ParamsFromLoginRules params = new ParamsFromLoginRules();
        boolean bParamsOK = prepareParamsFromLoginRules(params);
        if (!bParamsOK)
            return;

        String sAccountID = params.sAccountID;
        String sOfferID = params.sOfferID;
        int iBaseUnitSize = params.iBaseAmount;
        double dRateOffer = params.dAskRate;
        double dRatePrimary = dRateOffer + params.dPointSize * 25;
        double dRateSecondary1 = dRateOffer;
        double dRateSecondary2 = dRateOffer + params.dPointSize * 50;
        /*
         * Buy order on 10000 amoumrt
         * Sell order on 20000 amount
         * Buy order on 30000 amount
         *  "IF" order
         *      Sell order on 70000 amount
         *      Buy order on 60000 amount
         *      Buy order on 50000 amount
         */

        //// creating OTO command
        O2GRequestFactory factory = mSession.getRequestFactory();
        O2GValueMap valuemap = factory.createValueMap();
        valuemap.setString(O2GRequestParamsEnum.COMMAND, Constants.Commands.CreateOTO);

        //// create primary order
        valuemap.appendChild(createOrder(sAccountID, sOfferID, dRatePrimary, 1 * iBaseUnitSize,
                    Constants.Buy, Constants.Order.StopEntry, "FirstPrimary"));

        //// create 2 secondary orders
        valuemap.appendChild(createOrder(sAccountID, sOfferID, dRateSecondary1, 2 * iBaseUnitSize,
                    Constants.Sell, Constants.Order.StopEntry,  "SecondSecondary"));

        valuemap.appendChild(createOrder(sAccountID, sOfferID, dRateSecondary2, 3 * iBaseUnitSize,
                   Constants.Buy, Constants.Order.StopEntry, "ThirdSecondary" ));

        //// create secondary OTO group
        O2GValueMap valuemapSecondary = factory.createValueMap();
        valuemapSecondary.setString(O2GRequestParamsEnum.COMMAND, Constants.Commands.CreateOTO);

        //// create primary "IF" order
        valuemapSecondary.appendChild(createOrder(sAccountID, sOfferID, dRateSecondary1,
                    0, Constants.Sell, Constants.Order.StopEntry, "PrimaryIF"));

        //// create 3 secondary orders
            valuemapSecondary.appendChild(createOrder(sAccountID, sOfferID, dRatePrimary,
                                              7 * iBaseUnitSize, Constants.Sell, Constants.Order.LimitEntry, "Seventh"));

            valuemapSecondary.appendChild(createOrder(sAccountID, sOfferID, dRateSecondary2,
                                              6 * iBaseUnitSize, Constants.Buy, Constants.Order.StopEntry, "Sixth"));

            valuemapSecondary.appendChild(createOrder(sAccountID, sOfferID, dRateSecondary2,
                                              5 * iBaseUnitSize, Constants.Buy, Constants.Order.StopEntry, "Fifth"));

            valuemap.appendChild(valuemapSecondary);
            
            O2GRequest request = factory.createOrderRequest(valuemap);
            mSession.sendRequest(request);
    }

    private O2GValueMap createOrder(String sAccountID, String sOfferID, double dRate, int iAmount, String sBuySell, String sOrderType, String sCustomID)
    {
        O2GRequestFactory factory = mSession.getRequestFactory();
        O2GValueMap valuemap = factory.createValueMap();
        valuemap.setString(O2GRequestParamsEnum.COMMAND, Constants.Commands.CreateOrder);
        valuemap.setString(O2GRequestParamsEnum.ORDER_TYPE, sOrderType);
        valuemap.setString(O2GRequestParamsEnum.ACCOUNT_ID, sAccountID);
        valuemap.setString(O2GRequestParamsEnum.OFFER_ID, sOfferID);
        valuemap.setString(O2GRequestParamsEnum.BUY_SELL, sBuySell);
        valuemap.setInt(O2GRequestParamsEnum.AMOUNT, iAmount);
        valuemap.setDouble(O2GRequestParamsEnum.RATE, dRate);
        valuemap.setString(O2GRequestParamsEnum.CUSTOM_ID, sCustomID);
        return valuemap;
    }
       
    private boolean prepareParamsFromLoginRules(ParamsFromLoginRules params)
    {
        boolean bOK = false;
        O2GResponseReaderFactory factory = mSession.getResponseReaderFactory();
        O2GLoginRules loginRules = mSession.getLoginRules();
        // get first account from login
        O2GResponse accountResponse = loginRules.getTableRefeshResponse(O2GTable.ACCOUNTS);
        O2GAccountsTableResponseReader accountsReader = factory.createAccountsTableReader(accountResponse);
        O2GAccountRow account = accountsReader.getRow(0);
        // get account id
        params.sAccountID = account.getAccountID();
        // get base amount
        params.iBaseAmount = account.getBaseUnitSize();
        params.sOfferID = "";
        params.dAskRate = 0;
        params.dBidRate = 0;
        params.dPointSize = 0;
        // Get offers for EUR/USD
        O2GResponse offersResponse = loginRules.getTableRefeshResponse(O2GTable.OFFERS);
        O2GOffersTableResponseReader offersReader = factory.createOffersTableReader(offersResponse);
        for (int i = 0; i < offersReader.size(); i++)
        {
            O2GOfferRow offer = offersReader.getRow(i);
            if (offer.getInstrument().equals("EUR/USD"))
            {
                params.sOfferID = offer.getOfferID();
                params.dAskRate = offer.getAsk();
                params.dBidRate = offer.getBid();
                params.dPointSize = offer.getPointSize();
                bOK = true;
                break;
            }
        }
        return bOK;
    }


}