package setsubscription;

import com.fxcore2.Constants;
import java.util.concurrent.ConcurrentLinkedQueue;
import com.fxcore2.O2GRequest;
import com.fxcore2.O2GRequestFactory;
import com.fxcore2.O2GRequestParamsEnum;
import com.fxcore2.O2GResponse;
import com.fxcore2.O2GResponseReaderFactory;
import com.fxcore2.O2GSession;
import com.fxcore2.O2GValueMap;
import com.fxcore2.O2GLoginRules;
import com.fxcore2.O2GTable;
import com.fxcore2.O2GAccountsTableResponseReader;
import com.fxcore2.O2GAccountRow;
import com.fxcore2.O2GOfferRow;
import com.fxcore2.O2GOffersTableResponseReader;

public class EventListener {
    private SimpleLog mSimpleLog;
    private ConcurrentLinkedQueue<O2GResponse> mResponseQueue;
    private O2GSession mSession;
    private EventPipe mEventPipe;


    public EventListener(SimpleLog simpleLog, ConcurrentLinkedQueue<O2GResponse> responseQueue, O2GSession session, EventPipe eventPipe) {
        mSimpleLog = simpleLog;
        mResponseQueue = responseQueue;
        mSession = session;
        mEventPipe = eventPipe;
    }

    public void waitNextEvent() {
        boolean bContinue = true;
        while (bContinue) {
            EventType eEventType = mEventPipe.waitEvent();
            if (eEventType == EventType.STATUS) {
                this.onStatusChanged();
            } else if (eEventType == EventType.CONNECTED) {
                this.onConnected();
            } else if (eEventType == EventType.RESPONSE) {
                this.onResponse();
            } else {
                // "How to login" : Logout when you are done.
                mSession.logout();
                bContinue = false;
            }
        }
    }

    private void onResponse() {
        while (true) {
            if (mResponseQueue.isEmpty())
                break;

            O2GResponse response = mResponseQueue.poll();
        }
    }

    private void setSubscriptionStatus(String sStatus, String sOfferID)
    {
        O2GValueMap valueMap = mSession.getRequestFactory().createValueMap();
        valueMap.setString(O2GRequestParamsEnum.COMMAND, Constants.Commands.SetSubscriptionStatus);
        valueMap.setString(O2GRequestParamsEnum.SUBSCRIPTION_STATUS, sStatus);
        valueMap.setString(O2GRequestParamsEnum.OFFER_ID, sOfferID);

        O2GRequestFactory factory = mSession.getRequestFactory();
        O2GRequest request = factory.createOrderRequest(valueMap);
        Main.sSetSubscriptionStatusRequestID = request.getRequestId();
        mSession.sendRequest(request);
    }

    private void onConnected() {
        O2GLoginRules loginRules = mSession.getLoginRules();
        O2GResponse offersResponse = loginRules.getTableRefeshResponse(O2GTable.OFFERS);
        O2GResponseReaderFactory responseReaderFactory = mSession.getResponseReaderFactory();
        O2GOffersTableResponseReader offersReader = responseReaderFactory.createOffersTableReader(offersResponse);
        String sOfferID = "";
        boolean bFound = false;
        for (int k = 0; k < offersReader.size(); k++) // find an OfferID for the Instrument
        {
            O2GOfferRow offerRow = offersReader.getRow(k);
            if (offerRow.getInstrument().equals(Main.sInstrument))
            {
                sOfferID = offerRow.getOfferID();
                bFound = true;
            }
        }
        if (!bFound) {
            System.out.println("Cannot find an offerID for instrument " + Main.sInstrument);
            return;
        }
        setSubscriptionStatus(Main.sStatus, sOfferID);
    }

 
    private void onStatusChanged() {
    }

}